/*******************************************************************************
* File Name: `$INSTANCE_NAME`.c  
* Version `$CY_MAJOR_VERSION`.`$CY_MINOR_VERSION`
*
* Description:
*  This file contains the setup, control and status commands for the 
*  GraphicLCDIntf component.  
*
* Note: 
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "`$INSTANCE_NAME`.h"  
#include "`$INSTANCE_NAME`_fonts.h"
#include <stdlib.h>   // function abs

uint8 `$INSTANCE_NAME`_initVar = 0u;
uint16 `$INSTANCE_NAME`_MAX_X=0, `$INSTANCE_NAME`_MAX_Y = 0;
  

/*******************************************************************************
* Function Name: `$INSTANCE_NAME`_Enable
********************************************************************************
*
* Summary:
*  There is no need to reset or enable this component.  This component is 
*  naturally a slave that waits for the API calls in order to perform any 
*  function and once a request is made that request will always complete in 
*  finite time as long as a clock is provided. The only thing that is provided
*  by this API is setting the Command FIFO to drive at least half empty status
*  from the FIFO.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void `$INSTANCE_NAME`_Enable(void) `=ReentrantKeil($INSTANCE_NAME . "_Enable")`
{   
    /* There aren't anything to enable */
}


/*******************************************************************************
* Function Name: `$INSTANCE_NAME`_Init
********************************************************************************
*
* Summary:
*  Inits/Restores default GraphicLCDIntf configuration provided with customizer
*
* Parameters:
*  None.
*
* Return:
*  None.
* 
*******************************************************************************/
void `$INSTANCE_NAME`_Init(void) `=ReentrantKeil($INSTANCE_NAME . "_Init")`
{
    /* Set low and high pulse widths for read transaction */
    `$INSTANCE_NAME`_READ_LO_PULSE_REG = `$INSTANCE_NAME`_READ_LOW_PULSE;
    `$INSTANCE_NAME`_READ_HI_PULSE_REG = `$INSTANCE_NAME`_READ_HIGH_PULSE;
}


/*******************************************************************************
* Function Name: `$INSTANCE_NAME`_Start
********************************************************************************
*
* Summary:
*  Starts the GraphicLCDIntf interface.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  `$INSTANCE_NAME`_initVar - used to check initial configuration, modified on 
*  first function call.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void `$INSTANCE_NAME`_Start(void) `=ReentrantKeil($INSTANCE_NAME . "_Start")`
{
    if (`$INSTANCE_NAME`_initVar == 0u)
    {
        `$INSTANCE_NAME`_Init();
        `$INSTANCE_NAME`_initVar = 1u;
    }
      
    `$INSTANCE_NAME`_Enable();
    `$INSTANCE_NAME`_Setup(2u);  // start in landscape
    
}


/*******************************************************************************
* Function Name: `$INSTANCE_NAME`_Stop
********************************************************************************
*
* Summary:
*  Disables the GraphicLCDIntf interface.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void `$INSTANCE_NAME`_Stop(void) `=ReentrantKeil($INSTANCE_NAME . "_Stop")`
{
    /* There aren't anything to stop */
}


/*******************************************************************************
* Function Name: `$INSTANCE_NAME`_Write`$BitWidthReplacementString`
********************************************************************************
*
* Summary:
*  Initiates a write transaction on the `$BitWidthReplacementString`-bit parallel
*  interface.
*  The write is a posted write, so this function will return before the write 
*  has actually completed on the interface. If the command queue is full, this 
*  function will not return until space is availale to queue this write request.
*
* Parameters:
*  d_c:  Data(1) or Command(0) indication. Passed to the d_c pin.
*
*  data: Data sent on the do_msb[7:0] (most significant byte) 
*        and do_lsb[7:0] (least significant byte) pins. do_msb[7:0]
*        presents only for 16-bit interface.
*
* Return:
*  None.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void `$INSTANCE_NAME`_Write`$BitWidthReplacementString`(uint8 d_c, `$DataTypeReplacementString` wrData) \
                                                 `=ReentrantKeil($INSTANCE_NAME . "_Write`$BitWidthReplacementString`")`
{
    while((`$INSTANCE_NAME`_STATUS_REG & `$INSTANCE_NAME`_CMD_QUEUE_FULL) != 0u)
    {
            /* Command queue is full */
    }   
    `$INSTANCE_NAME`_CMD_FIFO_REG = d_c;
    #if (`$INSTANCE_NAME`_BUS_WIDTH == 16u)
        CY_SET_REG16(`$INSTANCE_NAME`_DATA_FIFO_PTR, wrData);
    #else /* 8-bit interface */
        `$INSTANCE_NAME`_DATA_FIFO_REG = wrData;
    #endif /* `$INSTANCE_NAME`_BUS_WIDTH == 16u */
}
	

/*******************************************************************************
* Function Name: `$INSTANCE_NAME`_WriteM`$BitWidthReplacementString`
********************************************************************************
*
* Summary:
*  Initiates multiple write transactions on the `$BitWidthReplacementString`-bit 
*  parallel interface. The write is a posted write, so this function will return
*  before the write has actually completed on the interface. If the command 
*  queue is full, this function will not return until space is availale to queue
*  this write request.
*
* Parameters:
*  d_c:  Data(1) or Command(0) indication. Passed to the d_c pin.
*
*  data: Pointer to the the first element of a data buffer sent on the do_msb[7:0]
*        (most significant byte) and do_lsb[7:0] (least significant byte) pins. 
*        do_msb[7:0] presents only for 16-bit interface.
*
*  num:  Number of words/bytes to write.
*
* Return:
*  None.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void `$INSTANCE_NAME`_WriteM`$BitWidthReplacementString`(uint8 d_c, `$DataTypeReplacementString`* wrData, uint16 num) \
                                                `=ReentrantKeil($INSTANCE_NAME . "_WriteM`$BitWidthReplacementString`")`
{
    for(; num; num--)
    {
        while((`$INSTANCE_NAME`_STATUS_REG & `$INSTANCE_NAME`_CMD_QUEUE_FULL) != 0u)
        {
            /* Command queue is full */
        }   
        `$INSTANCE_NAME`_CMD_FIFO_REG = d_c;  
 
        #if (`$INSTANCE_NAME`_BUS_WIDTH == 16u)
            CY_SET_REG16(`$INSTANCE_NAME`_DATA_FIFO_PTR, *wrData++);
        #else /* 8-bit interface */
            `$INSTANCE_NAME`_DATA_FIFO_REG = *wrData++;
        #endif /* `$INSTANCE_NAME`_BUS_WIDTH == 16u */
    }
}


/*******************************************************************************
* Function Name: `$INSTANCE_NAME`_Read`$BitWidthReplacementString`
********************************************************************************
*
* Summary:
*  Initiates a read transaction on the `$BitWidthReplacementString`-bit parallel interface.
*  The read will execute after all currently posted writes have completed. This 
*  function will wait until the read completes and then returns the read value.
*
* Parameters:
*  d_c: Data(1) or Command(0) indication. Passed to the d_c pin.
*
* Return:
*  16-bit read value from the di_msb[7:0] (most significant byte) and 
*  di_lsb[7:0] (least significant byte) pins for 16-bit interface or
*  8-bit read value from the di_lsb[7:0] for 8-bit interface.
*
* Reentrant:
*  No.
*
*******************************************************************************/
`$DataTypeReplacementString` `$INSTANCE_NAME`_Read`$BitWidthReplacementString`(uint8 d_c) \
                                                  `=ReentrantKeil($INSTANCE_NAME . "_Read`$BitWidthReplacementString`")`
{
    while((`$INSTANCE_NAME`_STATUS_REG & `$INSTANCE_NAME`_CMD_QUEUE_FULL) != 0u)
    {
        /* Command queue is full */
    }   
    `$INSTANCE_NAME`_CMD_FIFO_REG = d_c | `$INSTANCE_NAME`_READ_COMMAND;
    
    while((`$INSTANCE_NAME`_STATUS_REG & `$INSTANCE_NAME`_DATA_VALID) != `$INSTANCE_NAME`_DATA_VALID)
    {
        /* wait until input data are valid */    
    }
    
    #if (`$INSTANCE_NAME`_BUS_WIDTH == 8u)  /* 8-bit interface */
        return (`$INSTANCE_NAME`_DIN_LSB_DATA_REG);
    #else   /* 16-bit interface */
        return ((((uint16) `$INSTANCE_NAME`_DIN_MSB_DATA_REG) << 8u) | `$INSTANCE_NAME`_DIN_LSB_DATA_REG); 
    #endif /* `$INSTANCE_NAME`_BUS_WIDTH == 8u */
}

//************************************************************************************

//  Next is token from the TFT.c file

//************************************************************************************
uint16 constrain(uint16 a, uint16 b, uint16 c)
{
	if (a < b)  { return b; }
	if (c < a)	{ return c;	}
	else return a;
}

/**************************************************************************************
 *  Write Data to Index + Register
 *************************************************************************************/
void `$INSTANCE_NAME`_WriteRegData(uint8 Reg, uint16 Data)
{
    `$INSTANCE_NAME`_WriteIndex(Reg);
    `$INSTANCE_NAME`_WriteData(Data);
}

/**************************************************************************************
 *  Write Index  (Register #  -  RS = 0)
 *************************************************************************************/
void `$INSTANCE_NAME`_WriteIndex(uint8 Reg)
{
    `$INSTANCE_NAME`_Write`$BitWidthReplacementString`(0u, 0x00);   // Write to Index register
    `$INSTANCE_NAME`_Write`$BitWidthReplacementString`(0u, Reg);
}

/**************************************************************************************
 *  Write to Data Register  (RS = 1)
 *************************************************************************************/
void `$INSTANCE_NAME`_WriteData(uint16 Data)
{
	uint8 data1 = Data>>8;
	uint8 data2 = Data&0xff;
    
    `$INSTANCE_NAME`_Write`$BitWidthReplacementString`(1u, data1);  // Write data
    `$INSTANCE_NAME`_Write`$BitWidthReplacementString`(1u, data2);
}

//*************************************************************************************
void `$INSTANCE_NAME`_Setup(uint8 orient)
{
//************* Start Initial Sequence **********//
// VCI=2.8V
//************* Reset LCD Driver ****************//

    
//************* Start Initial Sequence **********//
    `$INSTANCE_NAME`_WriteRegData(0xE5, 0x8000); // Set the internal vcore voltage
    `$INSTANCE_NAME`_WriteRegData(0x00, 0x0001); // Start internal OSC.
    `$INSTANCE_NAME`_WriteRegData(0x01, 0x0100); // set SS and SM bit
    `$INSTANCE_NAME`_WriteRegData(0x02, 0x0700); // set 1 line inversion
    `$INSTANCE_NAME`_WriteRegData(0x03, 0x1030); // set GRAM write direction and BGR=1.
    `$INSTANCE_NAME`_WriteRegData(0x04, 0x0000); // Resize register
    `$INSTANCE_NAME`_WriteRegData(0x08, 0x0202); // set the back porch and front porch
    `$INSTANCE_NAME`_WriteRegData(0x09, 0x0000); // set non-display area refresh cycle ISC[3:0]
    `$INSTANCE_NAME`_WriteRegData(0x0A, 0x0000); // FMARK function
    `$INSTANCE_NAME`_WriteRegData(0x0C, 0x0000); // RGB interface setting
    `$INSTANCE_NAME`_WriteRegData(0x0D, 0x0000); // Frame marker Position
    `$INSTANCE_NAME`_WriteRegData(0x0F, 0x0000); // RGB interface polarity
//*************Power On sequence ****************//
    `$INSTANCE_NAME`_WriteRegData(0x10, 0x0000); // SAP, BT[3:0], AP, DSTB, SLP, STB
    `$INSTANCE_NAME`_WriteRegData(0x11, 0x0007); // DC1[2:0], DC0[2:0], VC[2:0]
    `$INSTANCE_NAME`_WriteRegData(0x12, 0x0000); // VREG1OUT voltage
    `$INSTANCE_NAME`_WriteRegData(0x13, 0x0000); // VDV[4:0] for VCOM amplitude
    CyDelay(200); // Dis-charge capacitor power voltage
    `$INSTANCE_NAME`_WriteRegData(0x10, 0x17B0); // SAP, BT[3:0], AP, DSTB, SLP, STB
    `$INSTANCE_NAME`_WriteRegData(0x11, 0x0037); // DC1[2:0], DC0[2:0], VC[2:0]
    CyDelay(50); // Delay 50ms
    `$INSTANCE_NAME`_WriteRegData(0x12, 0x013A); // VREG1OUT voltage
    CyDelay(50); // Delay 50ms
    `$INSTANCE_NAME`_WriteRegData(0x13, 0x1600); // VDV[4:0] for VCOM amplitude
    `$INSTANCE_NAME`_WriteRegData(0x29, 0x000C); // VCM[4:0] for VCOMH
    CyDelay(50);
    `$INSTANCE_NAME`_WriteRegData(0x20, 0x0000); // GRAM horizontal Address
    `$INSTANCE_NAME`_WriteRegData(0x21, 0x0000); // GRAM Vertical Address
// ----------- Adjust the Gamma Curve ----------//
    `$INSTANCE_NAME`_WriteRegData(0x30, 0x0504);
    `$INSTANCE_NAME`_WriteRegData(0x31, 0x0703);
    `$INSTANCE_NAME`_WriteRegData(0x32, 0x0702);
    `$INSTANCE_NAME`_WriteRegData(0x35, 0x0101);
    `$INSTANCE_NAME`_WriteRegData(0x36, 0x0A1F);
    `$INSTANCE_NAME`_WriteRegData(0x37, 0x0504);
    `$INSTANCE_NAME`_WriteRegData(0x38, 0x0003);
    `$INSTANCE_NAME`_WriteRegData(0x39, 0x0706);
    `$INSTANCE_NAME`_WriteRegData(0x3C, 0x0707);
    `$INSTANCE_NAME`_WriteRegData(0x3D, 0x091F);
//------------------ Set GRAM area ---------------//
    `$INSTANCE_NAME`_WriteRegData(0x50, 0x0000); // Horizontal GRAM Start Address
    `$INSTANCE_NAME`_WriteRegData(0x51, 0x00EF); // Horizontal GRAM End Address
    `$INSTANCE_NAME`_WriteRegData(0x52, 0x0000); // Vertical GRAM Start Address
    `$INSTANCE_NAME`_WriteRegData(0x53, 0x013F); // Vertical GRAM Start Address
    `$INSTANCE_NAME`_WriteRegData(0x60, 0x2700); // Gate Scan Line
    `$INSTANCE_NAME`_WriteRegData(0x61, 0x0001); // NDL,VLE, REV
    `$INSTANCE_NAME`_WriteRegData(0x6A, 0x0000); // set scrolling line
//-------------- Partial Display Control ---------//
    `$INSTANCE_NAME`_WriteRegData(0x80, 0x0000);
    `$INSTANCE_NAME`_WriteRegData(0x81, 0x0000);
    `$INSTANCE_NAME`_WriteRegData(0x82, 0x0000);
    `$INSTANCE_NAME`_WriteRegData(0x83, 0x0000);
    `$INSTANCE_NAME`_WriteRegData(0x84, 0x0000);
    `$INSTANCE_NAME`_WriteRegData(0x85, 0x0000);
//-------------- Panel Control -------------------//
    `$INSTANCE_NAME`_WriteRegData(0x90, 0x0010);
    `$INSTANCE_NAME`_WriteRegData(0x92, 0x0000);
    `$INSTANCE_NAME`_WriteRegData(0x93, 0x0003);
    `$INSTANCE_NAME`_WriteRegData(0x95, 0x0110);
    `$INSTANCE_NAME`_WriteRegData(0x97, 0x0000);
    `$INSTANCE_NAME`_WriteRegData(0x98, 0x0000);
    `$INSTANCE_NAME`_WriteRegData(0x07, 0x0173); // 262K color and display ON
    
    `$INSTANCE_NAME`_SetOrientation(orient);    // Orientation
}


//*************************************************************************************
void `$INSTANCE_NAME`_SetCol(uint16 StartCol,uint16 EndCol)
{
	`$INSTANCE_NAME`_WriteRegData(0x50, StartCol);     // Horizontal Start address
	`$INSTANCE_NAME`_WriteRegData(0x51, EndCol);       // Horizontal End address  
}
//*************************************************************************************
void `$INSTANCE_NAME`_SetRow(uint16 StartRow,uint16 EndRow)
{
	`$INSTANCE_NAME`_WriteRegData(0x50, StartRow);     // Horizontal Start address
	`$INSTANCE_NAME`_WriteRegData(0x51, EndRow);       // Horizontal End address  
}
//*************************************************************************************
void `$INSTANCE_NAME`_ClearScreen(uint16 bkColor)
{
	uint16 i=0;
    
	`$INSTANCE_NAME`_SetCol(0, `$INSTANCE_NAME`_MAX_X);
	`$INSTANCE_NAME`_SetRow(0, `$INSTANCE_NAME`_MAX_Y);
    
	`$INSTANCE_NAME`_WriteIndex(0x22);      // start to write to display ram

	for(i=0; i<19200; i++)
	{
		`$INSTANCE_NAME`_WriteData(bkColor);
		`$INSTANCE_NAME`_WriteData(bkColor);
		`$INSTANCE_NAME`_WriteData(bkColor);
		`$INSTANCE_NAME`_WriteData(bkColor);
	}
}
//*************************************************************************************
void `$INSTANCE_NAME`_FillScreen(uint16 XL, uint16 XR, uint16 YU, uint16 YD, uint16 color)
{
	unsigned long  XY=0;
	unsigned long i=0;
	color = ~color;
    
	if(XL > XR)
	{
		XL = XL^XR;
		XR = XL^XR;
		XL = XL^XR;
	}
	if(YU > YD)
	{
		YU = YU^YD;
		YD = YU^YD;
		YU = YU^YD;
	}
	XL = constrain(XL, `$INSTANCE_NAME`_MIN_X, `$INSTANCE_NAME`_MAX_X);
	XR = constrain(XR, `$INSTANCE_NAME`_MIN_X, `$INSTANCE_NAME`_MAX_X);
	YU = constrain(YU, `$INSTANCE_NAME`_MIN_Y, `$INSTANCE_NAME`_MAX_Y);
	YD = constrain(YD, `$INSTANCE_NAME`_MIN_Y, `$INSTANCE_NAME`_MAX_Y);

	XY = (XR-XL+1);
	XY = XY*(YD-YU+1);

	`$INSTANCE_NAME`_SetCol(XL, XR);
	`$INSTANCE_NAME`_SetRow(YU, YD);
    
	`$INSTANCE_NAME`_WriteIndex(0x22);      // start to write to display ram

	uint8 Hcolor = color>>8;
	uint8 Lcolor = color&0xff;
    
	for(i=0; i < XY; i++)
	{
		`$INSTANCE_NAME`_WriteData(Hcolor);
		`$INSTANCE_NAME`_WriteData(Lcolor);
	}

}

//*************************************************************************************
void `$INSTANCE_NAME`_SetXY(uint16 poX, uint16 poY)
{
	`$INSTANCE_NAME`_SetCol(poX, poX);
	`$INSTANCE_NAME`_SetRow(poY, poY);
}

//*************************************************************************************
void `$INSTANCE_NAME`_SetPixel(uint16 poX, uint16 poY,uint16 color)
{
	`$INSTANCE_NAME`_SetXY(poX, poY);
	`$INSTANCE_NAME`_WriteRegData(0x22, ~color);
}

//*************************************************************************************
void `$INSTANCE_NAME`_FillRectangle(uint16 poX, uint16 poY, uint16 length, uint16 width, uint16 color)
{
	`$INSTANCE_NAME`_FillScreen(poX, poX+length, poY, poY+width, color);
}

//*************************************************************************************
void  `$INSTANCE_NAME`_DrawHorizontalLine( uint16 poX, uint16 poY, uint16 length,uint16 color)
{
	int i=0;
	`$INSTANCE_NAME`_SetCol(poX, poX + length);
	`$INSTANCE_NAME`_SetRow(poY, poY);
    
	`$INSTANCE_NAME`_WriteIndex(0x22);
	for(i=0; i<length; i++)
	`$INSTANCE_NAME`_WriteData(~color);
}

//*************************************************************************************
void `$INSTANCE_NAME`_DrawLine( uint16 x0,uint16 y0,uint16 x1, uint16 y1,uint16 color)
{
	int x = x1-x0;
	int y = y1-y0;
	int dx = abs(x), sx = x0<x1 ? 1 : -1;
	int dy = -abs(y), sy = y0<y1 ? 1 : -1;
	int err = dx+dy, e2;                                                /* error value e_xy             */
	for (;;){                                                           /* loop                         */
		`$INSTANCE_NAME`_SetPixel(x0,y0,color);
		e2 = 2*err;
		if (e2 >= dy) {                                                 /* e_xy+e_x > 0                 */
			if (x0 == x1) break;
			err += dy; x0 += sx;
		}
		if (e2 <= dx) {                                                 /* e_xy+e_y < 0                 */
			if (y0 == y1) break;
			err += dx; y0 += sy;
		}
	}
}

//*************************************************************************************
void `$INSTANCE_NAME`_DrawVerticalLine( uint16 poX, uint16 poY, uint16 length,uint16 color)
{
	int i=0;
	`$INSTANCE_NAME`_SetCol(poX, poX);
	`$INSTANCE_NAME`_SetRow(poY, poY+length);
    
	`$INSTANCE_NAME`_WriteIndex(0x22);
	for(i=0; i<length; i++)
	`$INSTANCE_NAME`_WriteData(~color);
}

//*************************************************************************************
void `$INSTANCE_NAME`_DrawRectangle(uint16 poX, uint16 poY, uint16 length, uint16 width,uint16 color)
{
	`$INSTANCE_NAME`_DrawHorizontalLine(poX, poY, length, color);
	`$INSTANCE_NAME`_DrawHorizontalLine(poX, poY+width, length, color);
	`$INSTANCE_NAME`_DrawVerticalLine(poX, poY, width,color);
	`$INSTANCE_NAME`_DrawVerticalLine(poX + length, poY, width,color);
}
//*************************************************************************************
void `$INSTANCE_NAME`_DrawCircle(int poX, int poY, int r,uint16 color)
{
	int x = -r, y = 0, err = 2-2*r, e2;
	do {
		`$INSTANCE_NAME`_SetPixel(poX-x, poY+y,color);
		`$INSTANCE_NAME`_SetPixel(poX+x, poY+y,color);
		`$INSTANCE_NAME`_SetPixel(poX+x, poY-y,color);
		`$INSTANCE_NAME`_SetPixel(poX-x, poY-y,color);
		e2 = err;
		if (e2 <= y) {
			err += ++y*2+1;
			if (-x == y && e2 <= x) e2 = 0;
		}
		if (e2 > x) err += ++x*2+1;
	} while (x <= 0);
}
//*************************************************************************************
void `$INSTANCE_NAME`_FillCircle(int poX, int poY, int r,uint16 color)
{
	int x = -r, y = 0, err = 2-2*r, e2;
	do {
		`$INSTANCE_NAME`_DrawVerticalLine(poX-x, poY-y, 2*y, color);
		`$INSTANCE_NAME`_DrawVerticalLine(poX+x, poY-y, 2*y, color);

		e2 = err;
		if (e2 <= y) {
			err += ++y*2+1;
			if (-x == y && e2 <= x) e2 = 0;
		}
		if (e2 > x) err += ++x*2+1;
	} while (x <= 0);
}
//*************************************************************************************
void `$INSTANCE_NAME`_SetOrientation(uint8 orient)
{
	`$INSTANCE_NAME`_WriteIndex(0x03);          // Entry mode
    
	switch (orient)
	{
		case 0: `$INSTANCE_NAME`_WriteData(0x1030);    // Portrait Top-Down
				break;
		case 1: `$INSTANCE_NAME`_WriteData(0x1010);    // Portrait Bottom-Up
				break;
		case 2: `$INSTANCE_NAME`_WriteData(0x1018);    // Landscape Top-Down
				break;
		case 3: `$INSTANCE_NAME`_WriteData(0x1008);    // Landscape Bottom-Up
				break;
	}
	if (orient == 0 || orient == 2)
	{ `$INSTANCE_NAME`_MAX_X = 239; `$INSTANCE_NAME`_MAX_Y = 319; }
	else
	{ `$INSTANCE_NAME`_MAX_X = 319;	`$INSTANCE_NAME`_MAX_Y = 239; }
}
//*************************************************************************************
void `$INSTANCE_NAME`_DrawChar( uint8 ascii, uint16 poX, uint16 poY,uint16 size, uint16 fgcolor)
{
	int i = 0;   uint8 f =0 ;
	uint8 temp=0, k;
	if((ascii>=32)&&(ascii<=255))	{;}
	else	{ascii = '?'-32;}
	
	for (i = 0; i < `$INSTANCE_NAME`_FONT_X; i++ ) 
	{
		if ((ascii >= 0x20) && (ascii <= 0x7F))
		{
			temp = (Font16x16[ascii-0x20][i]);   // temp = (&Font16x16[ascii-0x20][i]);
		}
		else if ( ascii >= 0xC0 )
		{
			temp = (Font16x16[ascii-0x65][i]);  //  temp = (&Font16x16[ascii-0x65][i]);+++++++++++++++++++
		}
		k=i / 8;
		for(f =0 ; f < `$INSTANCE_NAME`_FONT_Y; f++)
		{
			if((temp>>f)&0x01)
			{
				if (size == 0)`$INSTANCE_NAME`_SetPixel(poX+i-(k*8), poY+f+(k*8),fgcolor);
				else `$INSTANCE_NAME`_FillRectangle(poX+i*size-(k*8)*size, poY+f*size+(k*8)*size, size, size, fgcolor);
			}
		}
	}
}

//*************************************************************************************
void `$INSTANCE_NAME`_DrawString(char *string,uint16 poX, uint16 poY, uint16 size,uint16 fgcolor)
{
	while(*string)
	{
		if((poX + `$INSTANCE_NAME`_FONT_SPACE) > `$INSTANCE_NAME`_MAX_X)		{poX = 1; poY = poY + `$INSTANCE_NAME`_FONT_X  *size;}
		
		`$INSTANCE_NAME`_DrawChar(*string, poX, poY, size, fgcolor);
		if (size > 0) poX += `$INSTANCE_NAME`_FONT_SPACE*size;
		else poX += `$INSTANCE_NAME`_FONT_SPACE;
		string++;
	}
}

//************************************************************************************++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void `$INSTANCE_NAME`_FillBitmap(uint16 XL, uint16 XR, uint16 YU, uint16 YD, unsigned short *Bitmap)
{
	unsigned long  XY=0;
	unsigned long i=0;
	XY = (XR-XL+1);
	XY = XY*(YD-YU+1);

	`$INSTANCE_NAME`_SetCol(XL,XR);
	`$INSTANCE_NAME`_SetRow(YU, YD);
    
	`$INSTANCE_NAME`_WriteIndex(0x22);          // GRAM

	for(i=0; i < XY; i++)
	{										    //color[i] =( ~color[i+1]);	
	    uint8 Hcolor = (~Bitmap[i])>>8;
	    uint8 Lcolor = (~Bitmap[i])&0xff;	
		`$INSTANCE_NAME`_WriteData(Hcolor);
		`$INSTANCE_NAME`_WriteData(Lcolor);
	}
}


//----------------------------------------------------------------------------------

/*
void Read_SD_Picture(uint16 XL, uint16 XR, uint16 YU, uint16 YD, unsigned short *FileName)
{
    char  fp;
    long  XY=0, i=0;
    XY = (XR-XL+1);
    XY = 2*XY*(YD-YU)+1;
    `$INSTANCE_NAME`_SetCol(XL,XR);
    `$INSTANCE_NAME`_SetRow(YU, YD);

    `$INSTANCE_NAME`_WriteIndex(0x22);      // GRAM

    fp = SDCard_fopen((uint8 *)FileName, (uint8 *)"r");

    while(! SDCard_feof(fp))                // Check for end of input file
    {
        extern uint8   SDCard_SectBuf1[];
        extern uint8   SDCard_SectBuf2[];

        SDCard_ReadFileSect(fp);            // Read sector

	    for(i=0; i < 256 ; i++)
		{
            if(XY<0)  break;
		    `$INSTANCE_NAME`_WriteData(~SDCard_SectBuf1[i]); XY--;
        }

	    for(i=0; i < 256; i++)
		{
            if(XY<0)  break;
		    `$INSTANCE_NAME`_WriteData(~SDCard_SectBuf2[i]); XY--;
        }
	
        SDCard_NextSect(fp);                // Advance to next read sector
}
	SDCard_fclose(fp); 
}
*/

//----------------------------------------------------------------------------------
void `$INSTANCE_NAME`_Graf(uint16 XL, uint16 YU, uint16 YD, uint8  Ydata , uint8 last)
{
	unsigned long XY=0;
	unsigned long i=0;
//	volatile char last;
	//XY = (XR-XL+1);
	XY = YD-YU+1;

	`$INSTANCE_NAME`_SetCol(XL,XL);
	`$INSTANCE_NAME`_SetRow(YU, YD);
    
	`$INSTANCE_NAME`_WriteIndex(0x22);          // GRAM

	for(i=0; i < XY; i++)
	{										    //color[i] =( ~color[i+1]);	
        //  uint8 Hcolor = 255*( (i<last )    );
	    //uint8 Hcolor = ~(255*( ((i>=last)&(i<=Ydata))|((i<=last)&(i>=Ydata))  ) );
        uint8 Hcolor =  (255*( ((i<last)|(i>Ydata))&((i>last)|(i<Ydata))  ) );
	    //uint8 Hcolor = 255*(Ydata!=i);//(~Bitmap[i])>>8;
	    uint8 Lcolor = 128;//(~Bitmap[i])&0xff;	
		`$INSTANCE_NAME`_WriteData(Hcolor);
		`$INSTANCE_NAME`_WriteData(Lcolor);
	}
    
    last =Ydata;
}


/* [] END OF FILE */
